unit frmmain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, TAGraph, TAStyles, TASeries, Forms, Controls,
  Graphics, Dialogs, EditBtn, StdCtrls, readstats;

type

  { TMainForm }

  TMainForm = class(TForm)
    Button1: TButton;
    CChars: TChart;
    CCharsBarSeries1: TBarSeries;
    CBRecurse: TCheckBox;
    DEDir: TDirectoryEdit;
    EExt: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { private declarations }
    FStats : TStats;
    FDirs : Integer;
    FCurrentDir : string;
    procedure ShowStats(ACount : Integer);
    procedure ThreadDone(Sender: TObject);
    Procedure DirDone(Const ADir : String);
  public
    { public declarations }

  end;

var
  MainForm: TMainForm;

implementation

{$R *.lfm}

{ TMainForm }
Type

  { TStatsThread }

  TStatsThread = Class(TThread)
  private
    FCurrentDir: String;
    FTotal: Integer;
    FDirectory : String;
    FExtensions : String;
    FRecurse : Boolean;
    FStats : PStats;
    FOnDir : TDirectoryCallBack;
    procedure DirDone(const ADir: String);
    procedure DoOnDir;
  Public
    Constructor Create(AStats : PSTats; ADirectory,AExtensions : String; Recurse : Boolean; AOnDestroy : TNotifyEvent; AOnDir : TDirectoryCallBack);
    Procedure Execute; override;
    Property Total : Integer Read FTotal Write FTotal;
    Property CurrentDir : String Read FCurrentDir;

  end;

{ TStatsThread }

constructor TStatsThread.Create(AStats: PSTats; ADirectory,
  AExtensions: String; Recurse: Boolean; AOnDestroy: TNotifyEvent; AOnDir : TDirectoryCallBack);
begin
  FDirectory:=ADirectory;
  FExtensions:=AExtensions;
  FRecurse:=Recurse;
  FStats:=ASTats;
  OnTerminate:=AOndestroy;
  FreeOnTerminate:=True;
  FOnDir:=AOnDir;
  Inherited Create(False);
end;

procedure TStatsThread.DoOnDir;

begin
  FOnDir(FCurrentDir);
end;

procedure TStatsThread.DirDone(Const ADir : String);

begin
  FCurrentDir:=ADir;
  If Assigned(FOnDir) then
    Synchronize(@DoOnDir);
end;

procedure TStatsThread.Execute;
begin
  FTotal:=GetStats(FStats^,FDirectory,FExtensions,FRecurse,@DirDone);
end;


procedure TMainForm.FormCreate(Sender: TObject);

Var
  B : TBarSeries;
  I : Integer;

begin
  DEDir.Directory:=ExtractFilePath(ParamStr(0));
  B:=CChars.Series[0] as TBarSeries;
  For I:=0 to 255 do
    begin
    B.AddXY(i,0);
    end;
end;

procedure TMainForm.ShowStats(ACount : Integer);

Var
  B : TBarSeries;
  I,C : Integer;

begin
  B:=CChars.Series[0] as TBarSeries;
  if ACount=-1 then
    C:=FStats[256]
  else
    C:=ACount;

  If (FCurrentDir<>'') then
    CChars.Title.Text[1]:=Format('%d files processed (%s)',[C,FCurrentDir])
  else
    CChars.Title.Text[1]:=Format('%d files processed',[C]);
  For I:=0 to 255 do
    begin
    B.SetYValue(I,FStats[i]);
    end;
  Application.ProcessMessages
end;

procedure TMainForm.ThreadDone(Sender: TObject);

begin
  FCurrentDir:='';
  ShowStats((Sender as TStatsThread).Total);
end;

procedure TMainForm.DirDone(const ADir: String);
begin
  FCurrentDir:=ExtractRelativePath(DEDir.Directory,ADir);
  ShowStats(-1);
end;

procedure TMainForm.Button1Click(Sender: TObject);

Var
  E,D : String;
  I : integer;

begin
  D:=IncludeTrailingPathDelimiter(DEDir.Directory);
  E:=EExt.Text;
  For I:=1 to Length(E) do
    If E[i]=' ' then E[I]:='.';
  E:='.'+E+'.';
  E:=StringReplace(E,'..','.',[rfReplaceAll]);
  FillWord(FStats,SizeOf(FStats) div 2,0);
  TStatsThread.Create(@FStats,D,E,CBRecurse.Checked,@ThreadDone,@DirDone);
end;

end.

