unit statsjob;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, readstats;

Type
  { TStatsJob }

  TStatsJob = Class(TObject)
  Private
    FExtensions: String;
    FOnDone: TNotifyEvent;
    FOnDir: TNotifyEvent;
    FRecurse: Boolean;
    FStartDir: String;
    FStats : PStats;
    FDirs : Integer;
    FCurrentDir : string;
    FTerminated: Boolean;
    FThread : TThread;
  Protected
    Procedure ThreadDone(Sender : TObject);
    Procedure DoExecute;
    procedure ShowDir;
    Procedure DoneDir(ADir : String);
    Procedure GetStats(Const ADirectory : String);
  Public
    Procedure Execute(UseThreads : Boolean = true);
    Procedure Terminate;
    Property Terminated : Boolean Read FTerminated;
    Property Stats : PStats Read FStats Write FStats;
    Property Dirs : Integer Read FDirs;
    Property CurrentDir : String Read FCurrentDir;
    Property OnDir : TNotifyEvent Read FOnDir Write FOndir;
    Property Extensions : String Read FExtensions Write FExtensions;
    Property StartDir : String Read FStartDir Write FStartDir;
    Property Recurse : Boolean Read FRecurse Write FRecurse;
    Property OnDone : TNotifyEvent Read FOnDone Write FOnDone;
  end;

  { TStatsThread }

  TStatsThread = Class(TThread)
  private
    FJob : TStatsJob;
  Public
    Constructor Create(AJob : TStatsJob);
    Procedure Execute; override;
  end;

implementation

{ TStatsJob }

procedure TStatsJob.ThreadDone(Sender : TObject);
begin
  FCurrentDir:='';
  If Assigned(OnDone) then
    OnDone(Self);
end;

procedure TStatsJob.DoExecute;
begin
  FCurrentDir:='';
  GetStats(StartDir);
end;

procedure TStatsJob.ShowDir;
begin
  If Assigned(FOnDir) then
    FOnDir(Self);
end;

procedure TStatsJob.DoneDir(ADir: String);
begin
  FCurrentDir:=ADir;
  TThread.Synchronize(FThread,@ShowDir);
end;

procedure TStatsJob.GetStats(const ADirectory: String);

Var
  Info : TSearchRec;
  E : String;

begin
  If FindFirst(ADirectory+'*.*',0,Info)=0 then
    try
      repeat
        E:=LowerCase(ExtractFileExt(Info.Name))+'.';
        If Pos(E,FExtensions)<>0 then
          UpdateStats(FStats^,ADirectory+Info.Name);
        Until (FindNext(Info)<>0) or Terminated;
      finally
        FindClose(Info);
      end;
  if Recurse and not terminated then
    If FindFirst(ADirectory+AllFilesMask,faDirectory,Info)=0 then
      try
        repeat
          if ((Info.Attr and faDirectory)<>0)
              and (Info.Name<>'..') and (info.name<>'.') then
            GetStats(Adirectory+Info.Name+PathDelim);
        Until (FindNext(Info)<>0) or Terminated;
      finally
        FindClose(Info);
      end;
  DoneDir(ADirectory);
end;

procedure TStatsJob.Execute(UseThreads : Boolean = true);
begin
  if UseThreads then
    FThread:=TStatsThread.Create(Self)
  else
    begin
    FThread:=Nil;
    DoExecute;
    ThreadDone(Self);
    end;
end;

procedure TStatsJob.Terminate;
begin
  FTerminated:=True;
end;

{ TStatsThread }

constructor TStatsThread.Create(AJob : TStatsJob);

begin
  FJob:=AJob;
  OnTerminate:=@FJob.ThreadDone;
  Inherited Create(False);
end;

procedure TStatsThread.Execute;
begin
  FJob.DoExecute
end;


end.

