unit frmmain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, TAGraph, TAStyles, TASeries, Forms, Controls,
  Graphics, Dialogs, EditBtn, StdCtrls, contnrs, statsjob, syncobjs, readstats;

type

  { TMainForm }

  TMainForm = class(TForm)
    Button1: TButton;
    BCancel: TButton;
    CChars: TChart;
    CCharsBarSeries1: TBarSeries;
    CBRecurse: TCheckBox;
    DEDir: TDirectoryEdit;
    EExt: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    procedure BCancelClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { private declarations }
    FStats : TStats;
    FJobSync : TCriticalSection;
    FSync : TCriticalSection;
    FJobs : TObjectList;
    procedure ShowStats(AJob : TStatsJob);
    procedure JobDone(Sender: TObject);
    Procedure DirDone(Sender: TObject);
  public
    { public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.lfm}

{ TMainForm }

procedure TMainForm.FormCreate(Sender: TObject);

Var
  B : TBarSeries;
  I : Integer;

begin
  DEDir.Directory:=ExtractFilePath(ParamStr(0));
  B:=CChars.Series[0] as TBarSeries;
  For I:=0 to 255 do
    begin
    B.AddXY(i,0);
    end;
  FSync:=TCriticalSection.Create;
  FJobs:=TObjectList.Create(True);
  FJobSync:=TCriticalSection.Create;
end;

procedure TMainForm.ShowStats(AJob : TStatsJob);

Var
  B : TBarSeries;
  I,C : Integer;

begin
  B:=CChars.Series[0] as TBarSeries;
  C:=FStats[256];
  If Assigned(AJob) and (AJob.CurrentDir<>'') then
    CChars.Title.Text[1]:=Format('%d jobs, %d files processed (%s)',[FJobs.Count,C,AJob.CurrentDir])
  else
    CChars.Title.Text[1]:=Format('%d jobs, %d files processed',[FJobs.Count,C]);
  C:=0;
  For I:=0 to 255 do
    C:=C+FStats[I];
  For I:=0 to 255 do
    begin
    B.SetYValue(I,FStats[i]/C*100);
    end;
  Application.ProcessMessages;
end;

procedure TMainForm.JobDone(Sender: TObject);

begin
  FJobSync.Enter;
  try
    FJobs.Remove(Sender);
  finally
    FJobSync.Leave;
  end;
  ShowStats(Nil);
end;

procedure TMainForm.DirDone(Sender : TObject);
begin
  ShowStats(TStatsJob(Sender));
  TStatsJob(Sender).ShowScheduled:=False;
end;

procedure TMainForm.Button1Click(Sender: TObject);

Var
  E,D : String;
  I : integer;
  J : TStatsJob;

begin
  J:=TStatsJob.Create;
  J.StartDir:=IncludeTrailingPathDelimiter(DEDir.Directory);
  J.Stats:=@FStats;
  E:=EExt.Text;
  For I:=1 to Length(E) do
    If E[i]=' ' then E[I]:='.';
  E:='.'+E+'.';
  E:=StringReplace(E,'..','.',[rfReplaceAll]);
  J.Extensions:=E;
  J.Recurse:=CBrecurse.Checked;
  J.OnDone:=@JobDone;
  J.OnDir:=@DirDone;
  FillWord(FStats,SizeOf(FStats) div 2,0);
  J.Sync:=FSync;
  J.Execute;
  FJobSync.Enter;
  try
    Fjobs.Add(J);
  finally
    FJobSync.Leave;
  end;
end;

procedure TMainForm.BCancelClick(Sender: TObject);

Var
  I : Integer;

begin
  If Assigned(FJobs) then
    begin
    FJobSync.Enter;
    try
      For I:=FJobs.Count-1 downto 0 do
        TStatsJob(Fjobs[i]).Terminate;
    finally
      FJobSync.Leave;
    end;
    end;
end;

end.

